/**
 *
 * Thu Mar 19 00:00:00 1998 GroupRenderer.java
 *
 * Copyright (C) 1998 - 2000  Joseph Garcia and Raymond Garcia
 *
 * This file is part of the Java Interactive Plotting Classes Library.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 **/
import java.awt.*;
import java.util.Vector;
import java.util.Observable;
import java.util.Observer;
import java.util.Date;
/**
  * Takes renderers and groups them into one graphics object.  This also
  * updates them correcly, all have equal weight.
  */
public class GroupRenderer extends DataRenderer
{
  private ObservationFunnel funn = new ObservationFunnel();
  public GroupRenderer(){
    ThreadPaint = true;
    funn.addObserver(this);
  }
  /**
    * Group member data consisting of a DataRenderer and a Boolean for each
    */
  protected Vector dat = new Vector(); //Each input is 2 objects: DataRenderer and Boolean
  private boolean keyon = false;
  /**
    * The Key renderer
    */
  public KeySubRenderer key = new KeySubRenderer();
  private long time = 0;
  /**
    * allows you to set a new KeySubRenderer to this object.  Null is not 
    * accepted.  Only one KeySubRenderer exists at this time.  Developers,
    * have fun.
    *
    * @param _key the new renderer
    */
  public synchronized void SetKey(KeySubRenderer _key){ if (_key != null) key = _key; }
  /**
    * Sets the font used by the key
    *
    * @param _f  the new Font
    */
  public synchronized void SetFont(Font _f){ if (_f != null) key.SetFont(_f); }
  /**
    * Sets whether or not the key is drawn
    *
    * @param b  drawn or not
    */
  public synchronized void SetKeyVisible(boolean b){  
    keyon = b; 
    setChanged(); notifyObservers();
  }
  /**
    * Toggles the key
    */
  public synchronized boolean ToggleKeyVisible(){
    keyon = !(keyon);
    setChanged(); notifyObservers();
    return keyon; 
  }
  /**
    * @return key is visible
    */
  public synchronized boolean GetKeyVisible(){ return keyon; }
  /**
    * Sets whether or not a renderer is drawn
    *
    * @param x the renderer in question
    * @param b drawing bool
    */
  public synchronized void SetVisible(DataRenderer x, boolean b) { 
    dat.setElementAt(new Boolean(b),dat.indexOf(x) +1);
    setChanged(); notifyObservers();
  }
  /**
    * @return  is drawn
    * @param x the renderer
    */
  public synchronized boolean GetVisible(DataRenderer x)
    {return ((Boolean)dat.elementAt(dat.indexOf(x) +1)).booleanValue();}
  /**
    * Sets the label used by the key for a renderer
    *
    * @param x the renderer
    * @param b the new label for x
    */
  public synchronized void SetLabel(DataRenderer x, String b) { 
    key.SetLabelAt(b,dat.indexOf(x)/2);
    setChanged(); notifyObservers();
  }
  /**
    * @return  label for renderer
    * @param x renderer
    */
  public synchronized String GetLabel (DataRenderer x)
    {return (key.GetLabelAt(dat.indexOf(x)/2));}
  /**
    * used for faster label getting using an index, not a renderer for a param.
    *
    * @return  label for renderer at index
    * @param x the index of a renderer
    */
  protected synchronized String GetLabel (int x)
    {
      if (key.GetLabelAt(x/2) != "")
	return (key.GetLabelAt(x/2));
      return ("(no label)");
    }
  /** used for faster draw-boolean getting using and index.
    *
    * @return is drawn
    * @param x index of renderer
    */
  protected synchronized boolean GetVisible(int x)
    {return ((Boolean)dat.elementAt(x+1)).booleanValue();}
  /**
    * abstractly inherited painting method. Loops to call all renderers in
    * group, passes on the parameter so all is on one graphics object.
    *
    * @param g the shared graphics object
    */
  public void draw( Graphics g )
    {
      Vector colors = new Vector();
      for (int x=0; x<dat.size(); x+=2)
	if (GetVisible(x)){
	  //System.out.println("Painting Graph #" + ((x/2)+1) +": " + GetLabel(x));
	  ((DataRenderer)dat.elementAt(x)).paint(g);
	  if (keyon){
	    colors.addElement(new Integer(x/2));
	    colors.addElement(g.getColor());
	    //key.draw(g,x/2,keyCount++);
	  }
	}
      for (int f=0; f<colors.size(); f+=2){
	g.setColor((Color)(colors.elementAt(f+1)));
	key.draw(g,((Integer)(colors.elementAt(f))).intValue(),f/2);
      }
    }
  /**
    * adds a data renderer to the group
    *
    * @param _in  the DataRenderer
    * @param name the renderer's label (for the key)
    */
public synchronized void AddInput(DataRenderer _in, String name)
    {
      if (_in != null) {
	if (! dat.contains(_in)) {
	  dat.addElement(_in);
	  dat.addElement(new Boolean(true));
	  key.AddLabel(name);
	  setCaching(_in.cacheEnabled() && cacheEnabled());
	  _in.addObserver(funn);
	  ThreadPaint = (ThreadPaint && (_in.ThreadPaint));
	}
      }
      //setChanged(); notifyObservers();
    }
  /**
    * removes a data renderer from the group
    *
    * @param out the renderer to remove
    */
  public synchronized void RemoveInput( DataRenderer out ) 
    {
      if (dat.contains(out)){
	key.RemoveLabelAt(dat.indexOf(out)/2);
	dat.removeElementAt(dat.indexOf(out)+1);
	dat.removeElement(out);
	out.deleteObserver(funn);
      }
    }
  /**
    * observing update method
    */
  public void update( Observable it, Object data )
    {
      //System.out.println("DBG: GR called");
      setChanged();notifyObservers();
    }
  public void Cancel(){
    for( int i=0; i<dat.size(); i+=2 ) 
      try{
	( (DataRenderer)dat.elementAt(i) ).Cancel(); 
      }catch(NullPointerException npe){}
    try{
      super.Cancel();
    }catch(NullPointerException npe){}
  }
  public double[] getExtents(){return null;}
  public int[] getDimensions() { return null; }
}

